.include "command_compatibility.inc"

;;; reset COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_reset:
    reset_str_flipflop
; Disable interrupts to prevent interrupts executing during initialization
    CLI
; Put microcontroller in initial state
    RJMP    reset

;;; clear_all COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_clear_all:
    reset_str_flipflop
; Clear data of all channels including start code
    RCALL   empty_tables
; Wait for new data
    RJMP    start

;;; clear_input COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_clear_input:
    reset_str_flipflop
; Clear data of all input channels including start code
    RCALL   empty_input
; Wait for new data
    RJMP    start

;;; clear_output COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_clear_output:
    reset_str_flipflop
; Clear data of all output channels including start code
    RCALL   empty_output
; Wait for new data
    RJMP    start

;;; defaults COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_defaults:
    reset_str_flipflop
; Reinitialise values to their firmware defaults
; (not to the values stored in EEPROM)
    RCALL   reset_parameters
; Wait for new data
    RJMP    start

;;; get_input_startcode COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_get_input_startcode:
; Fill up temporary pointer with address of input table (containing input start code)
    LDI     ptr_temp_h,high(input_min)
    LDI     ptr_temp_l,low(input_min)
; Read current input start code
    LD      sdata,Z
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; get_input_count COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_get_input_count:
; Read current input channel count value
    MOV     sdata,current_input_count
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; get_time_since_input COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_get_time_since_input:
; Check overflow flag of timer 1; if there was an overflow, the maximum possible value should be returned
    IN      temp1,TIFR
    SBRS    temp1,TOV1
    RJMP    get_time_since_input_readtimer
; Timer 1 overflow flag is set, so return the maximum possible value
    LDI     temp1,255
    LDI     temp2,255
    RJMP    get_time_since_input_output
get_time_since_input_readtimer:
; Timer 1 overflow flag is not set
; Store low byte of timer 1, part of indication of the time since the last DMX input break
    IN      temp2,TCNT1L
; Store high byte of timer 1
    IN      temp1,TCNT1H
get_time_since_input_output:
; Read high byte of time since break value
    MOV     sdata,temp1
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop

; Read low byte of time since break value
    MOV     sdata,temp2
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; get_mux_value COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_get_mux_value:
    CLR     sdata
; Check if the multiplexer inversion flag is set and
; use the correct code accordingly to collect a byte
; from the multiplexer pins
    SBRC    flags,mux_invert
    RJMP    get_mux_value_inverted
get_mux_value_normal:
    skip_if_not_mux_pin0
    SBR     sdata,1<<0
    skip_if_not_mux_pin1
    SBR     sdata,1<<1
    skip_if_not_mux_pin2
    SBR     sdata,1<<2
    skip_if_not_mux_pin3
    SBR     sdata,1<<3
    skip_if_not_mux_pin4
    SBR     sdata,1<<4
    skip_if_not_mux_pin5
    SBR     sdata,1<<5
    skip_if_not_mux_pin6
    SBR     sdata,1<<6
    skip_if_not_mux_pin7
    SBR     sdata,1<<7
    RJMP    get_mux_value_continue
get_mux_value_inverted:
    skip_if_mux_pin0
    SBR     sdata,1<<0
    skip_if_mux_pin1
    SBR     sdata,1<<1
    skip_if_mux_pin2
    SBR     sdata,1<<2
    skip_if_mux_pin3
    SBR     sdata,1<<3
    skip_if_mux_pin4
    SBR     sdata,1<<4
    skip_if_mux_pin5
    SBR     sdata,1<<5
    skip_if_mux_pin6
    SBR     sdata,1<<6
    skip_if_mux_pin7
    SBR     sdata,1<<7
get_mux_value_continue:
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; enable_input COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_enable_input:
    reset_str_flipflop
; Enable reception
    SBI     UCR,RXEN
; Wait for new data
    RJMP    start

;;; disable_input COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;

command_disable_input:
    reset_str_flipflop
; Disable reception
    CBI     UCR,RXEN
; Wait for new data
    RJMP    start

;;; enable_output COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_enable_output:
    reset_str_flipflop
; Only start transmitting if transmission is disabled, otherwise wait for new data
    SBIC    UCR,TXEN
    RJMP    start
; Transmit initial BREAK, MAB and initial value (start code)
    RCALL   transmit_break_mab
; Wait for new data
    RJMP    start

;;; disable_output COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;

command_disable_output:
    reset_str_flipflop
; Disable transmission
    CBI     UCR,TXEN
; Wait for new data
    RJMP    start

;;; input COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_input:
    reset_str_flipflop
; Retrieve character following to 'I'
    RCALL   wait_lpt_byte
; Check if the specified channel number is greater than input_channels - 2,
; if it is, return a value of 0 and don't attempt to read the value from
; the DMX input table, as only a limited number of input channels is supported
; by this firmware
    LDI     sdata,0
    CPI     rdata,input_channels - 1
    BRSH    command_input_immediate
; Fill up temporary pointer with address of input table (start code is skipped)
    LDI     ptr_temp_h,high(input_min + 1)
    LDI     ptr_temp_l,low(input_min + 1)
; Correct temporary pointer with specified channel number (start code could not be used)
    ADD     ptr_temp_l,rdata
    ADC     ptr_temp_h,dummy
; Read channel value from DMX input table
    LD      sdata,Z
command_input_immediate:
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; bulk_input COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_bulk_input:
    reset_str_flipflop
bulk_input_loop:
; Retrieve requested channel number (sync (character 255) = stop)
    RCALL   wait_lpt_byte
; Check whether stop was requested
    CPI     rdata,255
    BREQ    bulk_input_end
; Fill up temporary pointer with address of input table (start code is skipped)
    LDI     ptr_temp_h,high(input_min + 1)
    LDI     ptr_temp_l,low(input_min + 1)
; Correct temporary pointer with specified channel number (start code could not be used)
    ADD     ptr_temp_l,rdata
    ADC     ptr_temp_h,dummy
; Read channel value from DMX input table
    LD      sdata,Z
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Check that request is really character 255, otherwise stop
    CPI     rdata,255
    BRNE    bulk_input_end
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for next channel number
    RJMP    bulk_input_loop
bulk_input_end:
    reset_str_flipflop
    RJMP    start

;;; output COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_output:
    reset_str_flipflop
; Retrieve channel number
    RCALL   wait_lpt_byte
; Check if the specified channel number is equal to 255,
; if it is, ignore it, as only 255 output channels are supported
; by this firmware (excluding start code)
    CPI     rdata,255
    BREQ    command_output_finish
; Fill up temporary pointer with address of output table (start code is skipped)
    LDI     ptr_temp_h,high(output_min + 1)
    LDI     ptr_temp_l,low(output_min + 1)
; Correct temporary pointer with specified channel number (start code could not be specified)
    ADD     ptr_temp_l,rdata
    ADC     ptr_temp_h,dummy
    reset_str_flipflop
; Retrieve channel's new value
    RCALL   wait_lpt_byte
; Store channel value into DMX output table
    ST      Z,rdata
command_output_finish:
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; bulk_output COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_bulk_output:
    reset_str_flipflop
bulk_output_loop:
; Retrieve requested channel number (255 = stop)
    RCALL   wait_lpt_byte
; Store requested channel number
    MOV     temp1,rdata
    reset_str_flipflop
; Check whether stop was requested
    CPI     temp1,255
    BREQ    bulk_output_end
; Fill up temporary pointer with address of output table (start code is skipped)
    LDI     ptr_temp_h,high(output_min + 1)
    LDI     ptr_temp_l,low(output_min + 1)
; Correct temporary pointer with specified channel number (start code channel could not be specified)
    ADD     ptr_temp_l,rdata
    ADC     ptr_temp_h,dummy
; Retrieve new channel value
    RCALL   wait_lpt_byte
; Write channel value to DMX output table
    ST      Z,rdata
    reset_str_flipflop
; Wait for next channel number
    RJMP    bulk_output_loop
bulk_output_end:
    reset_str_flipflop
    RJMP    start

;;; set_mass_output_limits COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
command_set_mass_output_limits:
    reset_str_flipflop
; Wait for mass output minimum
    RCALL   wait_lpt_byte
; Temporarily store mass output minimum
    MOV     temp2,rdata
    reset_str_flipflop
; Wait for mass output maximum
    RCALL   wait_lpt_byte
; Temporarily store mass output maximum
    MOV     temp3,rdata
    reset_str_flipflop
; Check if the specified channel numbers are unequal to 255,
; if not, ignore them, as only 255 output channels are supported
; by this firmware (excluding start code)
    CPI     rdata,255
    BREQ    command_set_mass_output_limitsf
    CPI     rdata,255
    BREQ    command_set_mass_output_limitsf
; Check that mass_output_min <= mass_output_max (mass_output_max >= mass_output_min)
; Same as: check that not mass_output_min > mass_output_max (not mass_output_max < mass_output_min)
    CP      temp3,temp2
    BRLO    command_set_mass_output_limitsf
; Initialise mass output transfer minimum limit value to start of output table (skip start code)
    LDI     temp1,high(output_min + 1)
    MOV     mass_output_min_h,temp1
    LDI     temp1,low(output_min + 1)
    MOV     mass_output_min_l,temp1
; Correct mass output transfer minimum limit value with specified minimum (start code could not be specified)
    ADD     mass_output_min_l,temp2
    ADC     mass_output_min_h,dummy
; Initialise mass output transfer maximum limit value to start of output table (skip start code).
; The value must be increased by 1 as the limit check would be performed after the limit is passed
    LDI     temp1,high(output_min + 2)
    MOV     mass_output_max_h,temp1
    LDI     temp1,low(output_min + 2)
    MOV     mass_output_max_l,temp1
; Correct mass output transfer maximum limit value with specified maximum (start code could not be specified)
    ADD     mass_output_max_l,temp3
    ADC     mass_output_max_h,dummy
command_set_mass_output_limitsf:
    RJMP    start

;;; mass_output COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_mass_output:
    reset_str_flipflop
; Fill up temporary pointer with specified mass output minimum
    MOV     ptr_temp_h,mass_output_min_h
    MOV     ptr_temp_l,mass_output_min_l
; Fill up buffer variables with mass output maximum
    MOV     temp1,mass_output_max_h
    MOV     temp2,mass_output_max_l
mass_output_loop:
; Retrieve channel value
    RCALL   wait_lpt_byte
; Store channel value into DMX output table
    ST      Z+,rdata
    reset_str_flipflop
; Check mass output limit
    CP      ptr_temp_h,temp1
    CPC     ptr_temp_l,temp2
    BRNE    mass_output_loop
; Limit passed; wait for new data
    RJMP    start

;;; enable_mux COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_enable_mux:
    reset_str_flipflop
; Set mux flag (bit in flags register)
    SET
    BLD     flags,mux_status
; Wait for new data
    RJMP    start

;;; disable_mux COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;

command_disable_mux:
    reset_str_flipflop
; Unset override flag (bit in flags register)
    CLT
    BLD     flags,mux_status
; Wait for new data
    RJMP    start

;;; force_mux_off COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_force_mux_off:
    reset_str_flipflop
; Set force mux off flag (bit in flags register)
    SET
    BLD     flags,force_mux_off
; Wait for new data
    RJMP    start

;;; no_force_mux_off COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_no_force_mux_off:
    reset_str_flipflop
; Unset force mux off flag (bit in flags register)
    CLT
    BLD     flags,force_mux_off
; Wait for new data
    RJMP    start

;;; inverted_mux COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_inverted_mux:
    reset_str_flipflop
; Set mux invert flag (bit in flags register)
    SET
    BLD     flags,mux_invert
; Wait for new data
    RJMP    start

;;; non_inverted_mux COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_non_inverted_mux:
    reset_str_flipflop
; Unset mux invert flag (bit in flags register)
    CLT
    BLD     flags,mux_invert
; Wait for new data
    RJMP    start

;;; enable_merge COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_enable_merge:
    reset_str_flipflop
; Set merge (bit in flags register)
    SET
    BLD     flags,merge_status
; Wait for new data
    RJMP    start

;;; disable_merge COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_disable_merge:
    reset_str_flipflop
; Unset merge (bit in flags register)
    CLT
    BLD     flags,merge_status
; Wait for new data
    RJMP    start

;;; finish COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; The finish command should be called after an application
; is done using the interface, so that _ERR, _ACK, SEL and
; PE are set appropriately again for the compatibility mode
; to function.

command_finish:
    reset_str_flipflop
; Set _ERR, _ACK, SEL and PE properly for compatibility mode
    reset_err
    reset_ack
    reset_pe
    set_sel
; Wait for new data
    RJMP    start

;;; set_break_time COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_set_break_time:
    reset_str_flipflop
; Wait for break time byte
    RCALL   wait_lpt_byte
; Store the break time as 256 - <specified break time (microsec)> + 2
    LDI     temp1,255
    SUB     temp1,rdata
    INC     temp1
    INC     temp1
    INC     temp1
; Only store actual break time once fully calculated
    MOV     break_time,temp1
    reset_str_flipflop
    RJMP    start

;;; set_mab_time COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_set_mab_time:
    reset_str_flipflop
; Wait for mark after break (MAB) time byte
    RCALL   wait_lpt_byte
; Decrease specified MAB time by 1 before storing it
    DEC     rdata
; Store received byte as MAB time
    MOV     mab_time,rdata
    reset_str_flipflop
    RJMP    start

;;; enable_input_wildcard COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;

command_enable_input_wildcard:
    reset_str_flipflop
; Set input_wildcard flag (bit 1 in flags register)
    SET
    BLD     flags,input_wildcard
; Wait for new data
    RJMP    start

;;; disable_input_wildcard COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;

command_disable_input_wildcard:
    reset_str_flipflop
; Unset input_wildcard flag (bit 1 in flags register)
    CLT
    BLD     flags,input_wildcard
; Wait for new data
    RJMP    start

;;; set_input_startcode COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_set_input_startcode:
    reset_str_flipflop
; Wait for input start code value
    RCALL   wait_lpt_byte
; Store received byte as input start code
    MOV     input_startcode,rdata
    reset_str_flipflop
    RJMP    start

;;; set_output_startcode COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_set_output_startcode:
    reset_str_flipflop
; Wait for output start code value
    RCALL   wait_lpt_byte
; Fill up temporary pointer with address of output table (containing output start code)
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
; Store received byte as output start code
    ST      Z,rdata
    reset_str_flipflop
    RJMP    start

;;; set_output_count COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_set_output_count:
    reset_str_flipflop
; Wait for output count value
    RCALL   wait_lpt_byte
; Store received byte as output count
    MOV     output_count,rdata
    reset_str_flipflop
    RJMP    start

;;; set_mux_address COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_set_mux_address:
    reset_str_flipflop
; Wait for output count value
    RCALL   wait_lpt_byte
; Check that address doesn't exceed limit before updating it;
; the address must be <= output_channels - 8 - 1 (for start code)
    CPI     rdata,output_channels - 8 + 1 - 1
; Branch if same or higher
    BRSH    command_set_mux_address_finish
; Received byte is valid,
; so store it as address for channels multiplexed from secondary parallel connector
    MOV     mux_address,rdata
command_set_mux_address_finish:
    reset_str_flipflop
    RJMP    start

;;; read_eeprom COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_read_eeprom:
    reset_str_flipflop
    RCALL   read_eeprom
    RJMP    start

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.include "command_write_eeprom.inc"

;;; echo COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_echo:
    reset_str_flipflop
; Wait for output count value
    RCALL   wait_lpt_byte
; Store received byte in return buffer
    MOV     sdata,rdata
; Write first nibble to parallel port (the 4 least significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Swap sdata's nibble's so that the current least
; significant nibble gets discarded and the most significant
; one can be output to the parallel port.
    SWAP    sdata
; Wait for sync (character 255)
    RCALL   wait_lpt_byte
; Write second nibble to parallel port (the 4 most significant bits)
    RCALL   output_lpt_data
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
